//*********************************************************************
//    LSQR Inversion of AX=B
//      input parameters are explained in function show_help()
//
//    copyright:  (C) 2006 by Meijian An
//                Institute of Geomechanics, Chinese Academy of Geological Sciences
//
//********************************************************************** 
#ifdef _OPENMP
#include <omp.h>
#endif

#include <cstdio>
#include <cstring>
#include <cmath>
#include <vector>
#include <numeric>
#include <algorithm>
#include <functional>

#ifdef WIN32
#include "mystds.h"
#include "mysubs.h"
#include "nr3_my.h"
#else
#include "mystds.h"
#include "mysubs.h"
#include "nr3_my.h"
#endif

using namespace std;

struct Arguement
{
	long mx;                    //number of parameters in a solution vector (X).
	long nsols;                  //number of solutions.
	short ndim;                 //dimension number, e.g., 1 for 1D, 2 for 2D, 
	                            // 3 for 3D. for 3D, each point is represented by coordinates of x y z.
	vector<double> factors;     // multiple factors for each dimension
	struct data_range<double> resolution_length_range; 	//minimum and maximum of resolution length
	double resolution_length_error_margin; 	//resolution length error margin

	bool is_geography;           //if geographical coordinates is used.
	int normalization;           //normalize predicted solution in the same range with real model.

	string str_given_points;      //only invert for resolution matrix at given points
	bool is_recalc_dist;          //is re-calculate distance or temperarily keep distance values by an array

	string str_Xs_file;          //input file, a file with synthetic and inverted solutions
	string segment_symbol;   // segment seperator symbol for a multiple segment file. Default is '>'.

	string str_Resolution_file;  //output file, a file with statistic resolution length and alpha
	string str_Rmatrix_file;      //R matrix output file name. default is no to output R.
	
	bool no_openmp; //close openmp
};

int get_argument( struct Arguement &argu, int argc, char**argv );
long get_resolution_lengths( MatDoub &coords, MatDoub &sols_real, MatDoub &sols_calc, 
						    long mx, long nsols, short ndim, FILE *fp_res, FILE *fp_Rmatrix, 
							bool is_geography, struct data_range<double> resolution_length_range, 
							double resolution_length_error_margin, vector<long> & given_point_ids, bool is_recalc_dist);

int main(int argc, char *argv[])
{
	//get arguments
	struct Arguement argu;
	get_argument(argu, argc, argv);

#ifdef _OPENMP
	if(argu.no_openmp == false)
		set_openmp_parallel_process_num();
#endif

	//output file
	FILE *fp_res = openfile(argu.str_Resolution_file.c_str(), "w");
	FILE *fp_Rmatrix = argu.str_Rmatrix_file.empty()?NULL:openfile(argu.str_Rmatrix_file.c_str(), "w");

	vector<long> given_point_ids;
	if( argu.str_given_points.empty() == false)
	{
		FILE *fp_given_points = openfile(argu.str_given_points.c_str(), "r");
		char line[1024];
		long point_id;
		while( ! feof( fp_given_points ) )
		{
			if( fgets( line, 1024, fp_given_points ) == NULL) continue;
			if( sscanf( line, "%ld", &point_id) != 1 ) continue;
			if( point_id <= 0) continue;

			given_point_ids.push_back( point_id - 1 );//convert from 1
		}
		fclose( fp_given_points );
	}

	/************************************************************************************************************** 
	*         read solutions from a file with a format:
	*  x  (y)  (z)  sol_1_sythetic_model  sol_1_inverted_solution  sol_2_sythetic_model  sol_2_inverted_solution ...
	***************************************************************************************************************/
	MatDoub    coords(argu.mx, argu.ndim),
		       sols_real(argu.mx, argu.nsols),      //real solution, synthetic models, 
			   sols_calc(argu.mx, argu.nsols);      //calculated solution, inverted solutions

	//input file
	ifstream ifs_Xs;
	if(argu.str_Xs_file != "-" && argu.str_Xs_file != "") 
		openfile(&ifs_Xs, argu.str_Xs_file.c_str());
	
	istream &fin = (ifs_Xs.is_open() == false)?cin : ifs_Xs;

	string szLine;
	while( fin.eof() == false )
	{
		//read each parameters information
		long nx=0;
		VecDoub xmean_real(argu.mx, 0.0), xmean_calc(argu.mx, 0.0);
		while( (nx < argu.mx) && (fin.eof() == false) )
		{
			getline(fin, szLine);
			if( fin.bad() )continue;

			if( argu.segment_symbol.empty() == false )
			{
				if( szLine[0] == argu.segment_symbol[0] )
				{
					fprintf(fp_res, "%s", szLine.c_str());
					fprintf(fp_Rmatrix, "%s", szLine.c_str());
					SHOWerr1("%s", szLine.c_str());
					break;
				}
			}

			if( szLine.length() < (unsigned long)(2*argu.ndim) )
				continue;

			istringstream str_in(szLine);
			for( short idim = 0; idim < argu.ndim; idim ++ )
			{
				str_in >> coords[nx][idim];
				coords[nx][idim] *= argu.factors[idim];
			}

			if( str_in.fail() == true )
				continue;

			for( int isols=0; isols<argu.nsols; isols++ )
				str_in >> sols_real[nx][isols] >> sols_calc[nx][isols];

			if( str_in.fail() == true )
			{
				SHOWerr1("Warning: column Number < %ld\n", argu.ndim + argu.nsols *2);
				continue;
			}

			//calculate sum used to estimate average of xs.
			for( int isols=0; isols<argu.nsols; isols++ )
			{
				xmean_real[nx] += ( sols_real[nx][isols] / argu.nsols );
				xmean_calc[nx] += ( sols_calc[nx][isols] / argu.nsols );
			}

			nx ++;
		}

		if( nx <= 2 )
			continue;

		if( nx < argu.mx )
			SHOWerr2("Warning: solution parameter number (%ld) is smaller than expected %ld\n", nx, argu.mx);

		SHOW( " data information\n" );
		double xmean_real_all = 0.0, xmean_calc_all = 0.0;
		for( long iparam=0; iparam<nx; iparam++ )
		{
			xmean_real_all += ( xmean_real[iparam] / nx );
			xmean_calc_all += ( xmean_calc[iparam] / nx );
		}
		SHOW1( " mean_real_all = %lf\n", xmean_real_all);
		SHOW1( " mean_calc_all = %lf\n", xmean_calc_all);

		double mean_delta_real_all=0.0, mean_delta_calc_all=0.0;
		for( long iparam=0; iparam<nx; iparam++ )
		{
			for( int isols=0; isols<argu.nsols; isols++ )
			{
				mean_delta_real_all += pow(sols_real[iparam][isols] - xmean_real[iparam], 2);
				mean_delta_calc_all += pow(sols_calc[iparam][isols] - xmean_calc[iparam], 2);
			}
		}
		mean_delta_real_all = pow( mean_delta_real_all / nx / argu.nsols, 0.5);
		mean_delta_calc_all = pow( mean_delta_calc_all / nx / argu.nsols, 0.5);
		SHOW1( " mean_delta_real_all = %lf\n", mean_delta_real_all);
		SHOW1( " mean_delta_calc_all = %lf\n", mean_delta_calc_all);

		double ampl_ratio_all = mean_delta_calc_all / mean_delta_real_all;
		SHOW1( " diff_calc-real  = %lf\n", xmean_calc_all - xmean_real_all );
		SHOW1( " amplitude ratio = %lf\n", ampl_ratio_all);

		if( argu.normalization == 1) //normalization 1: using factor respectively for each parameter
		{
			SHOW( "  normalizing ... \n" );
			SHOW( "  ith   (mean_calc-mean_real)   amp_calc/amp_real\n" );
			for( long iparam=0; iparam<nx; iparam++ )
			{
				double delta_real=0.0, delta_calc=0.0;
				for( int isols=0; isols<argu.nsols; isols++ )
				{
					delta_real += pow(sols_real[iparam][isols] - xmean_real[iparam], 2);
					delta_calc += pow(sols_calc[iparam][isols] - xmean_calc[iparam], 2);
				}
				delta_real = pow( delta_real/argu.nsols, 0.5);
				delta_calc = pow( delta_calc/argu.nsols, 0.5);
			
				double ampl_ratio = delta_calc / delta_real;
				SHOW3( "%ld\t%lf\t%lf\n", iparam + 1, xmean_calc[iparam] - xmean_real[iparam], ampl_ratio);
				for( int isols=0; isols<argu.nsols; isols++ )
				{
					sols_calc[iparam][isols] = xmean_real[iparam] + (sols_calc[iparam][isols] - xmean_calc[iparam]) / ampl_ratio;
					//printf("%ld    %lf\n", isols+1, sols_calc[iparam][isols]);
				}
			}
		}
		else if(argu.normalization == 2) //normailzation 2: using a factor for all parameters
		{
			SHOW( "  normalizing ... \n" );
			for( long iparam=0; iparam<nx; iparam++ )
			{
				for( int isols=0; isols<argu.nsols; isols++ )
				{
					sols_calc[iparam][isols] = xmean_real_all + (sols_calc[iparam][isols] - xmean_calc_all) / ampl_ratio_all;
					//printf("%ld    %lf\n", isols+1, sols_calc[iparam][isols]);
				}
			}
		}

		fflush( NULL );

		//delete invalid ids of the given points
		for(long ipt = (given_point_ids.size() - 1); ipt >= 0; ipt--)
		{
			if(given_point_ids[ipt] < 0 || given_point_ids[ipt] >= nx)
				given_point_ids.erase(given_point_ids.begin() + ipt);
		}

		//calculate resolution lengths by simple grid searching
		get_resolution_lengths(coords, sols_real, sols_calc, nx, argu.nsols, argu.ndim, fp_res, 
								fp_Rmatrix, argu.is_geography, argu.resolution_length_range, 
								argu.resolution_length_error_margin, given_point_ids, argu.is_recalc_dist);

		if( argu.segment_symbol.empty() == true )
			break;
	}

	if( ifs_Xs.is_open() )
		ifs_Xs.close();
	if(fp_Rmatrix != NULL)
		fclose(fp_Rmatrix);
	fclose( fp_res );

	return 0;
}

int show_help(char *argv0 /*=argv[0]*/ )
{
	printf( "%s by Meijian An, 2014/10/08\n\n", argv0 );
	printf( "arguments:\n"
		    "    -h    show help\n"
		    "    -Xs=solution file name (input). If it is -Xs=-, read solutions from stdin\n"
		    "    -nsols=number of solutions.\n"
		    "    -mx=maximal number of parameters in a solution.\n"
			"    -given_points=a file with IDs of the parameters whose resolution lengths are calculated. Default is to calculate for all parameter.\n"
			"    -resolengths=min,max  resolution lengths. In default, they are half minimal distance and maximal distance of any two parameters.\n"
			"    -resolerr=?           resolution length error margin. In default, it is half minimal distance.\n"
		    "    -M=? or -M   seperator for a Multiple segment file.  The seperator character is '>' for -M\n"
			"    -normalization=?   normalization types. normalize predicted solution in the same range with real model. \n"
			"                   0: no normalize. (default)\n"
			"                   1: using a summary factor respectively the parameter itself.\n"
			"                   2: using a summary factor for all parameters.\n"
		    "    \n"
		    "    -dim=coordinate dimension number. e.g., 1(default) for 1D (x) system, or 2 for 2D (x, y), 3 for xyz.\n"
			"    -factors=factors (like 1.0,2.0,0.5) respectively multiple on coordinates (x, y, z). defaults factors=1,1,1,1,...\n"
			"    -geo   Using Geographic coordinate system. default is in Cartesian coordinate system. If it is set, -dim should be >=2, and only longitude/latitude are used.\n"
			"    -recalc_dist   All distance will be recalulated, which will use less memory but is inefficient.\n"
		    "    \n"
		    "    -Rs=resolution file (output). default is Rs.dat. \n"
		    "    -Rmatrix=file name to save normalized R matrix. At default, R matrix is not outputed. \n"
		    "    \n"
		    "    -log=log file name.\n"
		    );

	exit(0);
	return 1;
}

int get_argument(struct Arguement &argu, int argc, char**argv)
{
	if( get_switch( "-h", false, argc, argv ) || get_switch( "-help", false, argc, argv ) || argc<3 )
		return show_help( argv[0] );
 
 	//Get All arguement values.
	argu.is_geography        = get_switch( "-geo",               false,  argc, argv );
	argu.is_recalc_dist      = get_switch( "-recalc_dist",       false,  argc, argv );

	argu.nsols               = get_Parameter( "-nsols=",       long(0),  argc, argv );
	argu.mx                  = get_Parameter( "-mx=",         long(-1),  argc, argv );
	argu.ndim                = get_Parameter( "-dim=",        short(1),  argc, argv );

	struct data_range<double> default_range = {-1, -1};
	argu.resolution_length_range        = get_Parameter_range("-resolengths=", default_range, argc, argv );
	argu.resolution_length_error_margin = get_Parameter( "-resolerr=",         double(-1.0),  argc, argv );

	argu.normalization       = get_Parameter( "-normalization=",  int(0),  argc, argv );

	argu.str_given_points    = get_Parameter( "-given_points=", string(""),        argc, argv );
	argu.str_Xs_file         = get_Parameter( "-Xs=",              string(""),        argc, argv );
	argu.str_Resolution_file = get_Parameter( "-Rs=",              string("Rs.dat"),  argc, argv );
	argu.str_Rmatrix_file    = get_Parameter( "-Rmatrix=",         string(""),        argc, argv );

	//segment symbol
	argu.segment_symbol      = get_Parameter( "-M",string("NULL"),  argc, argv );
	if(argu.segment_symbol == "")
		argu.segment_symbol = ">";
	else if(argu.segment_symbol == "NULL")
		argu.segment_symbol = "";
	else
		argu.segment_symbol = argu.segment_symbol.substr(1, 1);

	//factors for each dimension.
	string factors           = get_Parameter( "-factors=",  string(""),  argc, argv );
	short idim;
	for( idim=0; idim<argu.ndim; idim++ )
	{
		double dtmp = 1.0;
		if( factors.empty() == false )
		{
			char szTmp[64];
			get_ith_column(factors.c_str(), szTmp, idim+1);
			strTrim(szTmp);
			if( strlen(szTmp) > 0 )
				sscanf(szTmp, "%lf", &dtmp);
		}
		argu.factors.push_back( dtmp );
	}

#ifdef _OPENMP
	argu.no_openmp      = get_switch("-no_openmp", false, argc, argv);
#else
	argu.no_openmp      = true;
#endif

	//output argument information
	string log_file  = get_Parameter("-log=", string(""), argc, argv);
	if(log_file.size() > 0) openLog(log_file.c_str());

	SHOW1( "%s options are:\n", argv[0] );
	SHOW1( "\t      solution file (-Xs) = %s\n",    argu.str_Xs_file.c_str() );
	SHOW1( "\tsolution parameter #(-mx) = %ld\n",   argu.mx );
	SHOW1( "\t  coordinate system is in : %s\n",    argu.is_geography ? "Geography coordinate system" : "Cartesian coordinate system" );
	SHOW1( "\t     recalculate distance : %s\n",    argu.is_recalc_dist ? "Yes. Inefficient option." : "No. Some memory will be used to recalculte distance, but can improve efficiency." );
	SHOW1( "\t      solutions #(-nsols) = %ld\n",   argu.nsols );
	SHOW1( "\t  normalize solution type = %d\n",    argu.normalization );
	SHOW1( "\tonly for the parameters in: %s\n",    argu.str_given_points.c_str());
	if(argu.resolution_length_range.min > 0)
		SHOW1( "\tminimal resolution length = %g\n", argu.resolution_length_range.min);
	if(argu.resolution_length_range.max > 0)
		SHOW1( "\tmaximal resolution length = %g\n", argu.resolution_length_range.max);
	if(argu.resolution_length_error_margin > 0)
		SHOW1( "\tresolution length error margin = %g\n", argu.resolution_length_error_margin);

	if(argu.resolution_length_error_margin > 0 && argu.resolution_length_range.max > 0 && argu.resolution_length_range.min > 0)
		SHOWerr2("  ** minimum No of Samples = %d for 95%s confidence level, according to your searched range and error margin. Normally the value is dependent real problem.\n", int(pow(1.96 * (argu.resolution_length_range.max - argu.resolution_length_range.min) * 0.29/ argu.resolution_length_error_margin, 2)), "%");

	SHOW1( "\t    resolution file (-Rs) = %s\n", argu.str_Resolution_file.c_str() );
	SHOW1( "\t         save R matrix in = %s\n", (argu.str_Rmatrix_file.empty()==false) ? argu.str_Rmatrix_file.c_str() : "(no output)" );

	SHOW( "\n" );
	SHOW1( "\tcoordinate dimsion (-dim) = %d\n", argu.ndim );
	for( idim=0; idim<argu.ndim; idim++ )
	{
		if( argu.factors[idim] != 1.0 )
			SHOW2( "\t     factor for %dth dimension = %g\n", idim+1, argu.factors[idim] );
	}

	if(	argu.segment_symbol.empty() == false )
		SHOW1( "\t     Multiple segment sepertor: %s\n", argu.segment_symbol.c_str() );

	if(	(argu.str_Xs_file.empty() == true) || argu.nsols < 1 || argu.mx < 2 )
	{
		SHOW( "\tError: -nsols, -mx and -Xs should be set!\n" );
		return show_help( argv[0] );
	}

	fflush( NULL );
	return 1;
}

double get_distance(long &ix, long &jx, MatDoub &coords, short &ndim, bool &is_geography)
{
	if(ix == jx)
		return 0.0;

	double dist = 0.0;
	if( (ndim >=2) && (is_geography == true) )
	{
		double az, baz, gcarc;
		GetDisAz( coords[ix][1], coords[ix][0], coords[jx][1], coords[jx][0], &az, &baz, &dist, &gcarc );
	}
	else
		dist = getDistance( coords[ix], coords[jx], ndim );
	return dist;
}

/************************************************************************************************************** 
*         estimate resolution length for all solution parameters
***************************************************************************************************************/
long get_resolution_lengths( MatDoub &coords, MatDoub &sols_real, MatDoub &sols_calc, 
						    long mx, long nsols, short ndim, FILE *fp_res, FILE *fp_Rmatrix, 
							bool is_geography, struct data_range<double> resolution_length_range, 
							double resolution_length_error_margin, vector<long> & given_point_ids, bool is_recalc_dist )
{
	long mx_given_points = given_point_ids.size();
	long mx_expect = ((mx_given_points > 0) ? mx_given_points : mx );

	long isols, ix, jx;
	short idim;
	double dist; // distances for a parameter to the other;

	//set default resoluton length range and error
	if(resolution_length_error_margin <= 0 || resolution_length_range.max <= 0 || resolution_length_range.min <= 0)
	{
		double dist_min = numeric_limits<double>::max(), 
			   dist_max = -1.0;

		//estimate min/max distance only from one 50th of distances, to save runing time.
		int idelta_xx = int(mx_expect/50);
		if(idelta_xx < 20) idelta_xx = 1;

		for(long ixx = 0; ixx < mx_expect; ixx += idelta_xx)
		{
			if( mx_given_points > 0 )
				ix = given_point_ids[ixx];
			else
				ix = ixx;

			for( jx = ix+1; jx < mx; jx ++ )
			{
				dist = get_distance(ix, jx, coords, ndim, is_geography);
				if(dist > dist_max) dist_max = dist;
				if(dist < dist_min) dist_min = dist;
			}
		}

		resolution_length_error_margin = dist_min/2.0;
		if(resolution_length_range.min < resolution_length_error_margin)
			resolution_length_range.min = resolution_length_error_margin;
		if(resolution_length_range.max < resolution_length_error_margin)
			resolution_length_range.max = dist_max / 2.0;
	}

	SHOWerr2("  ** searched %ld from %ld parameters.\n", mx_expect, mx);
	SHOWerr2("  ** searched range: from %g to %g.\n", resolution_length_range.min, resolution_length_range.max);
	SHOWerr1("  **   error margin: %g.\n", resolution_length_error_margin);
	SHOWerr("  Note: The above three values influence computation efficiency.\n");

	VecFloat length_best( mx, 0.0 ), 
			 misfit_best( mx ),
			 alpha(       mx ),
			 alpha_ref(   mx ) ;

	SHOWerr("    grid searching ...\n");

	int nthreads = 1;
#ifdef _OPENMP
	nthreads = omp_get_max_threads();
#endif

	float *dist_mem = NULL;
	float **dist_arrays = NULL;
	if( is_recalc_dist == false )
	{
		dist_mem = new float[mx * nthreads];
		if(dist_mem != NULL)
		{
			dist_arrays = new float*[nthreads];
			for(int i = 0; i < nthreads; i++)
				dist_arrays[i] = dist_mem + mx * i;
		}
	}

	#pragma omp parallel for default(shared) private(isols, idim, ix, jx) schedule(dynamic, 50)
	for(long ixx = 0; ixx < mx_expect; ixx ++ )
	{
		if( mx_given_points > 0 )
			ix = given_point_ids[ixx];
		else
			ix = ixx;

		int ithread = 0;
#ifdef _OPENMP
		ithread = omp_get_thread_num();
		if( ithread >= nthreads )
		{
			SHOWerr2("Error: ithread (%d) >= nthread (%d)", ithread, nthreads);
			exit(1);
		}
#endif
		if( dist_arrays != NULL)
		{
			for( jx = 0; jx < mx; jx ++ )
				dist_arrays[ithread][jx] = get_distance(ix, jx, coords, ndim, is_geography);
		}
		/************************************************************************************************************** 
		*		find best resolution length for a parameter by Exhaustive grid search or enumeration.
		***************************************************************************************************************/
		misfit_best[ix] = 1e20; //misfit for best resolution length
		length_best[ix] = -1;

		vector<double> sol_forward(      nsols,  0.0 ),
					   sol_forward_best( nsols, -1.0e20 );

		//evaluate all possible resolution-length values for the parameter
		for( double resol_length = resolution_length_range.min; resol_length <= resolution_length_range.max; resol_length += resolution_length_error_margin )
		{
			double dist_limit = resol_length * 4; //normalized guassion function value will be too small (< 0.00002) when dist >= dist_limit;
			double misfit = 0.0;
			for( isols = 0; isols < nsols; isols ++ )
			{
				/*************************************************************************************
				*    Normalized projection from real model to calculated solution:
				*        sol_forward_normalized_i = sum( R_i_j * sol_real_j ) / sum( R_i_j )
				*    and also should be:
				*        sol_forward_normalized_i = sol_calc_i
				***************************************************************************************/
				sol_forward[isols] = 0.0;
				double alpha_sum = 0.0;
				for( jx = 0; jx < mx; jx ++ )
				{
					if(is_recalc_dist == false && dist_arrays != NULL)
						dist = dist_arrays[ithread][jx]; 
					else
						dist = get_distance(ix, jx, coords, ndim, is_geography);

					if(dist > dist_limit)
						continue;

					double R_ij_f      = exp( (-1) * pow(dist, 2) /(pow(resol_length/1.17, 2) * 2.0));
					sol_forward[isols] += ( R_ij_f * sols_real[jx][isols] );
					alpha_sum          += R_ij_f;
				}
				double sol_forward_normalized = sol_forward[isols] / alpha_sum;
				//misfit += pow( sol_forward_normalized - sols_calc[ix][isols], 2)/nsols;
				misfit += fabs( sol_forward_normalized - sols_calc[ix][isols])/nsols;
			}

			//SHOW3(" %d   %lf   %f\n", ix, misfit_best[ix], misfit);
			if( misfit_best[ix] >= misfit )
			{
				length_best[ix]  = resol_length;
				misfit_best[ix]  = misfit;
				sol_forward_best = sol_forward;
			}
			//fprintf(fp_res, "%.1lf\t%.1lf\t%.2lf\t%.1lf\t%ld\t%.1lf\n", gcarc_best, fabs(ab_best[1]/ab_best[0]),  ab_best[0], ab_best[1], msol_used_best, misfit_best);
		}

		alpha_ref[ix] = alpha[ix] = 0.0;
		for( isols = 0; isols < nsols; isols++ )
		{
			alpha_ref[ix] += (sols_calc[ix][isols] / sols_real[ix][isols]);
			alpha[ix]     += (sols_calc[ix][isols] / sol_forward_best[isols]);
		}
		alpha_ref[ix] /= nsols;
		alpha[ix]     /= nsols;

		//only output in master thread
		if( (ix % 10 == 0) && (ithread == 0) )
		{
			SHOWerr3("master thread: %ld / %ld from %ld\n", ixx+1, mx_expect, mx);
			fflush( NULL );
		}

		//delete []dist_array;
	}

	if(dist_arrays != NULL)
	{
		delete []dist_arrays;
		delete []dist_mem;
	}

	//*************************************************************************************
	//   output resolution length and matrix
	//*************************************************************************************
	SHOWerr("    Output resolution ...\n");

	fprintf(fp_res, "## ");
	for( idim = 0; idim < ndim; idim ++ )
		fprintf(fp_res, "%c\t", 'x' + idim);
	fprintf(fp_res, "resolution_length  alpha_ref  alpha  misfit\n");

	for( long ixx = 0; ixx < mx_expect; ixx ++ )
	{
		if( mx_given_points > 0 )
			ix = given_point_ids[ixx];
		else
			ix = ixx;

		for( idim = 0; idim < ndim; idim ++ )
			fprintf(fp_res, "%lf\t", coords[ix][idim]);
		fprintf(fp_res, "%.2lf\t%.2lf\t%.2lf\t%.2lf\n", length_best[ix], alpha_ref[ix], alpha[ix], misfit_best[ix]);
	}
	SHOWerr("\n");

	if(fp_Rmatrix != NULL)
	{
		for( long ixx = 0; ixx < mx_expect; ixx ++ )
		{
			if( mx_given_points > 0 )
				ix = given_point_ids[ixx];
			else
				ix = ixx;

			for( jx = 0; jx < mx; jx ++ )
			{
				dist = get_distance(ix, jx, coords, ndim, is_geography);
				
				//output normalized resolution matrix
				double R_ij = exp( (-1.0) * pow(dist, 2) /(pow(length_best[ix]/1.17, 2) * 2.0));
				//double R_ij_r = alpha[ix] * R_ij;

				fprintf(fp_Rmatrix, "%lf\t", R_ij);
			}
			fprintf(fp_Rmatrix, "\n");
		}		
	}
	
	return mx_expect;
}
